@@ -472,7 +472,7 @@ class S3UtilsTest(unittest.TestCase):
             s3_push_text(client, text, bucket, key, encoding=encoding)
             self.assertEqual(s3_pull_text(client, bucket, key, encoding=encoding), text)
 
-    def test_s3_estimate_archive_ranged_requests(self):
+    def test_s3_archive_listfile(self):
         with (
             moto.mock_aws(),
             moto_server_endpoint_url() as endpoint_url,
@@ -486,30 +486,50 @@ class S3UtilsTest(unittest.TestCase):
                            "s3utils_archive")
 
             local_root = os.path.join(resources_directory, "unittest", "s3utils")
+            local_members = [os.path.relpath(file_path, local_root) for file_path in listfile(local_root)]
 
-            members = [os.path.relpath(file_path, local_root) for file_path in listfile(local_root)]
+            for archive_url, members in itertools.product(
+                ["s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
+                 "s3://dummy-bucket/s3utils_archive/archive.compressed.zip"],
+                [local_members, None],
+            ):
+                archive_size, member_zip_infos, missed_members = s3_archive_listfile(client, archive_url, members)
 
-            ranged_total_size, archive_size, missing_members = s3_estimate_archive_ranged_requests(
-                client,
-                "s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
-                members,
-                central_directory_overhead=0,
-            )
+                self.assertEqual(archive_size - sum(info.compress_size for info in member_zip_infos), 2470)
+                self.assertEqual(set(info.filename for info in member_zip_infos), set(local_members))
+                self.assertEqual(len(missed_members), 0)
 
-            self.assertEqual(archive_size - ranged_total_size, 1190)
-            self.assertEqual(len(missing_members), 0)
+            for archive_url, (members, members_expect) in itertools.product(
+                ["s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
+                 "s3://dummy-bucket/s3utils_archive/archive.compressed.zip",
+                 ],
+                [(local_members, local_members),
+                 (None, local_members),
+                 (["dir.baz/", "dir.foo/"],
+                  [os.path.relpath(file_path, local_root)
+                   for file_path in
+                   listfile(os.path.join(local_root, "dir.baz")) + listfile(os.path.join(local_root, "dir.foo"))],
+                  ),
+                 (["dir.foo/"],
+                  [os.path.relpath(file_path, local_root)
+                   for file_path in listfile(os.path.join(local_root, "dir.foo"))],
+                  ),
+                 (["dir.foo/"] * 3,  # Duplicate entries, the result should repeat them as well
+                  [os.path.relpath(file_path, local_root)
+                   for file_path in (listfile(os.path.join(local_root, "dir.foo")) * 3)],
+                  ),
+                 (["dir.foo/dir.foo.bar/"],
+                  [os.path.relpath(file_path, local_root)
+                   for file_path in listfile(os.path.join(local_root, "dir.foo", "dir.foo.bar"))],
+                  ),
+                 ],
+            ):
+                archive_size, member_zip_infos, missed_members = s3_archive_listfile(client, archive_url, members)
 
-            ranged_total_size, archive_size, missing_members = s3_estimate_archive_ranged_requests(
-                client,
-                "s3://dummy-bucket/s3utils_archive/archive.compressed.zip",
-                members,
-                central_directory_overhead=0,
-            )
+                self.assertEqual(sorted(list(info.filename for info in member_zip_infos)), sorted(members_expect))
+                self.assertEqual(len(missed_members), 0)
 
-            self.assertEqual(archive_size - ranged_total_size, 1190)
-            self.assertEqual(len(missing_members), 0)
-
-    def test_s3_open_archive_member(self):
+    def test_s3_archive_open_member(self):
         with (
             moto.mock_aws(),
             moto_server_endpoint_url() as endpoint_url,
@@ -523,22 +543,22 @@ class S3UtilsTest(unittest.TestCase):
                            "s3utils_archive")
 
             local_root = os.path.join(resources_directory, "unittest", "s3utils")
-
-            members = [os.path.relpath(file_path, local_root) for file_path in listfile(local_root)]
+            local_members = [os.path.relpath(file_path, local_root) for file_path in listfile(local_root)]
 
             for archive_url, mode in itertools.product(
                 ["s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
-                 "s3://dummy-bucket/s3utils_archive/archive.compressed.zip"],
+                 "s3://dummy-bucket/s3utils_archive/archive.compressed.zip",
+                 ],
                 ["r", "rb"],
             ):
-                for member in members:
+                for local_member in local_members:
                     with (
-                        open(os.path.join(local_root, member), mode) as local_fh,
-                        s3_open_archive_member(client, archive_url, member, mode) as s3_fh,
+                        open(os.path.join(local_root, local_member), mode) as local_fh,
+                        s3_archive_open_member(client, archive_url, local_member, mode) as s3_fh,
                     ):
                         self.assertEqual(local_fh.read(), s3_fh.read())
 
-    def test_s3_open_archive_members(self):
+    def test_s3_archive_open_members(self):
         with (
             moto.mock_aws(),
             moto_server_endpoint_url() as endpoint_url,
@@ -552,17 +572,18 @@ class S3UtilsTest(unittest.TestCase):
                            "s3utils_archive")
 
             local_root = os.path.join(resources_directory, "unittest", "s3utils")
+            local_members = [os.path.relpath(file_path, local_root) for file_path in listfile(local_root)]
 
-            members = [os.path.relpath(file_path, local_root) for file_path in listfile(local_root)]
-
-            for archive_url, mode, threshold, use_chunked_reads in itertools.product(
+            for archive_url, members, mode, threshold, use_chunked_reads in itertools.product(
                 ["s3://dummy-bucket/s3utils_archive/archive.uncompressed.zip",
-                 "s3://dummy-bucket/s3utils_archive/archive.compressed.zip"],
+                 "s3://dummy-bucket/s3utils_archive/archive.compressed.zip",
+                 ],
+                [local_members, None],
                 ["r", "rb"],
                 [1.0, 0.0],
                 [False, True],
             ):
-                for member, opener in s3_open_archive_members(
+                for member, opener in s3_archive_open_members(
                     client,
                     archive_url,
                     members,
